/***************************************/
/* class selection                     */
/*                                     */
/* holds information about a selection */
/* of wires/devices                    */
/*                                     */
/* Andreas Rostin                      */
/* 14.08.1999                          */
/***************************************/
#ifndef __SELECTIONCLASS
#define __SELECTIONCLASS

#include <qrect.h>
class XDeviceNet;
#include <xdevice.h>
#include <xwire.h>
#include "list.h"

// selection status
#define SEL_EMPTY	0
#define SEL_START	1
#define SEL_EXISTS	2
#define SEL_MOVE	3

class Selection
{
public:
	Selection();
	~Selection();

	int getStatus();
	void setNet(XDeviceNet *);

	// single object selection
	void add(XDevice *);
	void remove(XDevice *);
	int contains(XDevice *);
	void add(XWire *);
	void remove(XWire *);
	int contains(XWire *);
	void addTemp(XDevice *);
	void removeTemp(XDevice *);
	void fixTemp(XDevice *);
	int isTemp(XDevice *);

	// group selection
	void beginFrame(QPoint);
	void changeFrame(QPoint);
	void endFrame();
	int onFrame(QPoint);

	// moving selection
	void beginMove(QPoint);
	void moveFrame(QPoint, QPainter *);
	void endMove();
	int hasMoved();

	// copy selection
	void cut(QPainter *, XDeviceNet *);
	void copy(XDeviceNet *);
	void at(QPoint);
	void paste(QPainter *, XDeviceNet *);

	// remove selection
	void remove(QPainter *, int);
	void remove();

	// drawing methods (for the selection frame, not the content)
	void erase(QPainter *);
	void draw(QPainter *);

private:
	inline void setFrame(QPoint);
	inline void setFrame();
	inline QPoint *align(QPoint *);

	QPoint corner1;
	QRect frame;
	QRect old_frame;
	QPoint pt_move_start;

	// current status of a group selection, 0: begin must follow, 1: move or end must follow
	int selstat;

	int frame_moved;
	XDevice *tempdev;

	// objects a selection can contain
	list<XDevice> dev;
	list<XWire> wire;

	// buffer which contains the last pasted circuit
	static QString copy_buffer;
	// position to paste at
	QPoint paste_at;

	XDeviceNet *net;
};

#endif
